/* The following KLUDGE needs to be defined, because it seems
 * that the tapdance code does not support more than 9 defined
 * tapdance keys.  If you try to give it more, the software jumps
 * off the deep end and behaves erratic right after rebooting.
 */
#if 1
#define KLUDGE 1      /* Comment it out, to compile properly */
#endif

#include QMK_KEYBOARD_H

#include "quantum.h"

// Defines names for use in layer keycodes and the keymap
enum layer_names {
    _BASE,
    _NUMKEYPAD,
    _HEXKEYPAD,
    _FN,
    _FNPAGE,
    _MOUSEMOVE,
    _SCROLLWHEEL,
    _SYSTEM,
};

#ifdef ACTION_TAP_DANCE_DOUBLE
#include "action.h"
#include "process_keycode/process_tap_dance.h"

enum {
  UNTAPPED = 0,
  SINGLE_TAP,
  SINGLE_HOLD,
  DOUBLE_TAP,
  DOUBLE_HOLD,
  DOUBLE_SINGLE_TAP, //send two single taps
  TRIPLE_TAP,
  TRIPLE_HOLD,
  MORE_TAPS
};

typedef struct {
  uint16_t kc1;
  uint16_t kc2;
  int state;
} qk_tap_dance_multi_role_t;

#define HOLD_TERM 300 // in ms

#define ACTION_TAP_DANCE_MULTI_ROLE(KC1, KC2, TAPPINGTERM)   \
        { .fn = {qk_tap_dance_multi_role_on_each_tap, \
	         qk_tap_dance_multi_role_finished,    \
		 qk_tap_dance_multi_role_reset},      \
   .custom_tapping_term = TAPPINGTERM,		      \
   .user_data = (void *)&((qk_tap_dance_multi_role_t) \
                           { .kc1 = KC1, .kc2 = KC2,  \
                             .state = UNTAPPED}), }


void qk_tap_dance_multi_role_on_each_tap(qk_tap_dance_state_t *state, void *user_data) {
    qk_tap_dance_multi_role_t *pair = (qk_tap_dance_multi_role_t *)user_data;

    if (state->count > 1)
      register_code(pair->kc1);
}

// To activate SINGLE_HOLD, you will need to hold for 200ms first.
// This tap dance favours keys that are used frequently in typing like 'f'
void qk_tap_dance_multi_role_finished(qk_tap_dance_state_t *state, void *user_data) {
    qk_tap_dance_multi_role_t *pair = (qk_tap_dance_multi_role_t *)user_data;

  switch (state->count) {
    case 1:
      //If count = 1, and it has been interrupted - it doesn't matter if it is pressed or not: Send SINGLE_TAP
      if (state->interrupted) {
        //     if (!state->pressed) return SINGLE_TAP;
        //need "permissive hold" here.
        //     else return SINGLE_HOLD;
        //If the interrupting key is released before the tap-dance key, then it is a single HOLD
        //However, if the tap-dance key is released first, then it is a single TAP
        //But how to get access to the state of the interrupting key????
        pair->state = SINGLE_TAP;
      } else
        pair->state = state->pressed ? SINGLE_HOLD : SINGLE_TAP;
      break;
    case 2:
               // If count = 2, and it has been interrupted - assume that user is trying
	       // to type the letter associated with single tap.
      pair->state = state->interrupted ? DOUBLE_SINGLE_TAP
                  : state->pressed ? DOUBLE_HOLD : DOUBLE_TAP;
      break;
    case 3:
      pair->state = state->interrupted || !state->pressed ? TRIPLE_TAP : TRIPLE_HOLD;
      break;
    default:
      pair->state = MORE_TAPS;
      break;
  }

  switch (pair->state) {
    default:
    case SINGLE_TAP: register_code(pair->kc1); break;
    case SINGLE_HOLD: register_code(pair->kc2); unregister_code(pair->kc2); break;
  }
}

void qk_tap_dance_multi_role_reset(qk_tap_dance_state_t *state, void *user_data) {
    qk_tap_dance_multi_role_t *pair = (qk_tap_dance_multi_role_t *)user_data;

  switch (pair->state) {
    default:
    case SINGLE_TAP: unregister_code(pair->kc1); break;
    case SINGLE_HOLD: break;
  }
  pair->state = UNTAPPED;
}

//TD Declarations
enum {
_TD_F1 = 0,
_TD_F2,
_TD_F3,
_TD_F4,
_TD_F5,
_TD_F6,
_TD_F7,
#ifndef KLUDGE
_TD_F8,
_TD_F9,
_TD_F10,
#endif
_TD_F11,
_TD_F12,
};

#define TD_F1 TD(_TD_F1)
#define TD_F2 TD(_TD_F2)
#define TD_F3 TD(_TD_F3)
#define TD_F4 TD(_TD_F4)
#define TD_F5 TD(_TD_F5)
#define TD_F6 TD(_TD_F6)
#define TD_F7 TD(_TD_F7)

#ifndef KLUDGE
#define TD_F8 TD(_TD_F8)
#define TD_F9 TD(_TD_F9)
#define TD_F10 TD(_TD_F10)
#else
#define TD_F8 KC_8
#define TD_F9 KC_9
#define TD_F10 KC_0
#endif

#define TD_F11 TD(_TD_F11)
#define TD_F12 TD(_TD_F12)

// Tapdance definitions. Tap Dance F Keys.
qk_tap_dance_action_t tap_dance_actions[] = {
  [_TD_F1] = ACTION_TAP_DANCE_MULTI_ROLE(KC_1, KC_F1, HOLD_TERM),
  [_TD_F2] = ACTION_TAP_DANCE_MULTI_ROLE(KC_2, KC_F2, HOLD_TERM),
  [_TD_F3] = ACTION_TAP_DANCE_MULTI_ROLE(KC_3, KC_F3, HOLD_TERM),
  [_TD_F4] = ACTION_TAP_DANCE_MULTI_ROLE(KC_4, KC_F4, HOLD_TERM),
  [_TD_F5] = ACTION_TAP_DANCE_MULTI_ROLE(KC_5, KC_F5, HOLD_TERM),
  [_TD_F6] = ACTION_TAP_DANCE_MULTI_ROLE(KC_6, KC_F6, HOLD_TERM),
  [_TD_F7] = ACTION_TAP_DANCE_MULTI_ROLE(KC_7, KC_F7, HOLD_TERM),
#ifndef KLUDGE
  [_TD_F8] = ACTION_TAP_DANCE_MULTI_ROLE(KC_8, KC_F8, HOLD_TERM),
  [_TD_F9] = ACTION_TAP_DANCE_MULTI_ROLE(KC_9, KC_F9, HOLD_TERM),
  [_TD_F10] = ACTION_TAP_DANCE_MULTI_ROLE(KC_0, KC_F10, HOLD_TERM),
#endif
  [_TD_F11] = ACTION_TAP_DANCE_MULTI_ROLE(KC_MINS, KC_F11, HOLD_TERM),
  [_TD_F12] = ACTION_TAP_DANCE_MULTI_ROLE(KC_EQL, KC_F12, HOLD_TERM),
};

#else

#define TD_F1 KC_1
#define TD_F2 KC_2
#define TD_F3 KC_3
#define TD_F4 KC_4
#define TD_F5 KC_5
#define TD_F6 KC_6
#define TD_F7 KC_7
#define TD_F8 KC_8
#define TD_F9 KC_9
#define TD_F10 KC_0
#define TD_F11 KC_MINS
#define TD_F12 KC_EQL

#endif // ACTION_TAP_DANCE_DOUBLE

#include "json2c.h"
